@@ -1,7 +1,7 @@
 /* -*-mode:c; c-style:k&r; c-basic-offset:4; -*- */
 /* Balsa E-Mail Client
  *
- * Copyright (C) 1997-2013 Stuart Parmenter and others,
+ * Copyright (C) 1997-2016 Stuart Parmenter and others,
  *                         See the file AUTHORS for a list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /*
@@ -41,29 +39,9 @@
 
 #include "address-book.h"
 
-#define LIBBALSA_TYPE_ADDRESS_BOOK_GPE		(libbalsa_address_book_gpe_get_type())
-#define LIBBALSA_ADDRESS_BOOK_GPE(obj)		(G_TYPE_CHECK_INSTANCE_CAST(obj, LIBBALSA_TYPE_ADDRESS_BOOK_GPE, LibBalsaAddressBookGpe))
-#define LIBBALSA_ADDRESS_BOOK_GPE_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST (klass, LIBBALSA_TYPE_ADDRESS_BOOK_GPE, LibBalsaAddressBookGpeClass))
-#define LIBBALSA_IS_ADDRESS_BOOK_GPE(obj)		(G_TYPE_CHECK_INSTANCE_TYPE(obj, LIBBALSA_TYPE_ADDRESS_BOOK_GPE))
-#define LIBBALSA_IS_ADDRESS_BOOK_GPE_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE (klass, LIBBALSA_TYPE_ADDRESS_BOOK_GPE))
-
-typedef struct _LibBalsaAddressBookGpe LibBalsaAddressBookGpe;
-typedef struct _LibBalsaAddressBookGpeClass LibBalsaAddressBookGpeClass;
-
-struct _LibBalsaAddressBookGpe {
-    LibBalsaAddressBook parent;
-#ifdef HAVE_SQLITE3
-    sqlite3 *db;
-#else                           /* HAVE_SQLITE3 */
-    sqlite *db;
-#endif                          /* HAVE_SQLITE3 */
-};
-
-struct _LibBalsaAddressBookGpeClass {
-    LibBalsaAddressBookClass parent_class;
-};
-
-GType libbalsa_address_book_gpe_get_type(void);
+#define LIBBALSA_TYPE_ADDRESS_BOOK_GPE (libbalsa_address_book_gpe_get_type())
+G_DECLARE_FINAL_TYPE(LibBalsaAddressBookGpe, libbalsa_address_book_gpe,
+        LIBBALSA, ADDRESS_BOOK_GPE, LibBalsaAddressBook)
 
 LibBalsaAddressBook *libbalsa_address_book_gpe_new(const gchar *name);
 
