@@ -36,13 +36,21 @@
 #include "gmime-multipart-crypt.h"
 
 
+#ifdef G_LOG_DOMAIN
+#  undef G_LOG_DOMAIN
+#endif
+#define G_LOG_DOMAIN "crypto"
+
+
 /**
  * sign_prepare:
  * @mime_part: MIME part
  *
  * Prepare a part (and all subparts) to be signed. To do this we need
  * to set the encoding of all parts (that are not already encoded to
- * either QP or Base64) to QP.
+ * either QP or Base64 or 7-bit) to QP.
+ *
+ * Ref: RFC 3156, sect. 3.
  **/
 static void
 sign_prepare(GMimeObject * mime_part)
@@ -72,7 +80,7 @@ sign_prepare(GMimeObject * mime_part)
 	sign_prepare(subpart);
     } else {
 	encoding = g_mime_part_get_content_encoding(GMIME_PART(mime_part));
-	if (encoding != GMIME_CONTENT_ENCODING_BASE64)
+	if ((encoding != GMIME_CONTENT_ENCODING_BASE64) && (encoding != GMIME_CONTENT_ENCODING_7BIT))
 	    g_mime_part_set_content_encoding(GMIME_PART(mime_part),
 					     GMIME_CONTENT_ENCODING_QUOTEDPRINTABLE);
     }
@@ -228,13 +236,11 @@ g_mime_gpgme_mps_verify(GMimeMultipartSigned * mps, GError ** error)
     if (protocol) {
 	if (g_ascii_strcasecmp("application/pgp-signature", protocol) == 0)
 	    crypto_prot = GPGME_PROTOCOL_OpenPGP;
-#if defined(HAVE_SMIME)
 	else if (g_ascii_strcasecmp
 		 ("application/pkcs7-signature", protocol) == 0
 		 || g_ascii_strcasecmp("application/x-pkcs7-signature",
 				       protocol) == 0)
 	    crypto_prot = GPGME_PROTOCOL_CMS;
-#endif
 	else
 	    crypto_prot = GPGME_PROTOCOL_UNKNOWN;
     } else
@@ -438,7 +444,7 @@ g_mime_gpgme_mpe_decrypt(GMimeMultipartEncrypted * mpe,
     g_return_val_if_fail(GMIME_IS_MULTIPART_ENCRYPTED(mpe), NULL);
 
     if (signature && *signature) {
-	g_object_unref(G_OBJECT(*signature));
+	g_object_unref(*signature);
 	*signature = NULL;
     }
 
@@ -524,17 +530,17 @@ g_mime_gpgme_mpe_decrypt(GMimeMultipartEncrypted * mpe,
 	g_set_error(err, GMIME_ERROR, GMIME_ERROR_PARSE_ERROR, "%s",
 		    _
 		    ("Cannot decrypt multipart/encrypted part: failed to parse decrypted content"));
-	g_object_unref(G_OBJECT(sigstat));
+	g_object_unref(sigstat);
 	return NULL;
     }
 
 
     /* cache the decrypted part */
     if (signature) {
-	if (sigstat->status != GPG_ERR_NOT_SIGNED)
+	if (g_mime_gpgme_sigstat_status(sigstat) != GPG_ERR_NOT_SIGNED)
 	    *signature = sigstat;
 	else
-	    g_object_unref(G_OBJECT(sigstat));
+	    g_object_unref(sigstat);
     }
 
     return decrypted;
