@@ -375,7 +375,7 @@ libbalsa_is_cert_known(GTlsCertificate      *cert,
 			gchar *pem_data;
 			FILE *fd;
 
-			g_object_get(G_OBJECT(cert), "certificate-pem", &pem_data, NULL);
+			g_object_get(cert, "certificate-pem", &pem_data, NULL);
 			fd = fopen(cert_file, "a");
 			if (fd != NULL) {
 				fputs(pem_data, fd);
@@ -429,11 +429,11 @@ ask_cert_real(void *data)
                                          _("_Reject"), GTK_RESPONSE_CANCEL,
                                          NULL);
     gtk_window_set_role(GTK_WINDOW(dialog), "tls_cert_dialog");
-    g_object_get(G_OBJECT(acd->certificate), "certificate", &cert_der, NULL);
+    g_object_get(acd->certificate, "certificate", &cert_der, NULL);
     gcr_cert = gcr_simple_certificate_new(cert_der->data, cert_der->len);
     g_byte_array_unref(cert_der);
     cert_widget = GTK_WIDGET(gcr_certificate_widget_new(gcr_cert));
-    g_object_unref(G_OBJECT(gcr_cert));
+    g_object_unref(gcr_cert);
 
     str = g_string_new("");
     g_string_printf(str, _("<big><b>Authenticity of this certificate "
@@ -478,7 +478,7 @@ get_gnutls_cert(GTlsCertificate *cert)
     if (gnutls_res == GNUTLS_E_SUCCESS) {
     	GByteArray *cert_der;
 
-    	g_object_get(G_OBJECT(cert), "certificate", &cert_der, NULL);
+        g_object_get(cert, "certificate", &cert_der, NULL);
     	if (cert_der != NULL) {
     		gnutls_datum_t data;
 
@@ -659,59 +659,6 @@ libbalsa_am_i_subthread(void)
     return g_thread_self() != main_thread_id;
 }
 
-
-#include "libbalsa_private.h"	/* for prototypes */
-static GMutex mailbox_mutex;
-static GCond  mailbox_cond;
-
-/* Lock/unlock a mailbox; no argument checking--we'll assume the caller
- * took care of that. 
- */
-#define LIBBALSA_DEBUG_THREADS FALSE
-void
-libbalsa_lock_mailbox(LibBalsaMailbox * mailbox)
-{
-	GThread *thread_id = g_thread_self();
-
-    g_mutex_lock(&mailbox_mutex);
-
-    if (mailbox->thread_id && mailbox->thread_id != thread_id)
-        while (mailbox->lock)
-            g_cond_wait(&mailbox_cond, &mailbox_mutex);
-
-    /* We'll assume that no-one would destroy a mailbox while we've been
-     * trying to lock it. If they have, we have larger problems than
-     * this reference! */
-    mailbox->lock++;
-    mailbox->thread_id = thread_id;
-
-    g_mutex_unlock(&mailbox_mutex);
-}
-
-void
-libbalsa_unlock_mailbox(LibBalsaMailbox * mailbox)
-{
-	GThread *self;
-
-    self = g_thread_self();
-
-    g_mutex_lock(&mailbox_mutex);
-
-    if (mailbox->lock == 0 || self != mailbox->thread_id) {
-	g_warning("Not holding mailbox lock!!!");
-        g_mutex_unlock(&mailbox_mutex);
-	return;
-    }
-
-    if(--mailbox->lock == 0) {
-        mailbox->thread_id = 0;
-        g_cond_broadcast(&mailbox_cond);
-    }
-
-    g_mutex_unlock(&mailbox_mutex);
-}
-
-
 /* Initialized by the front end. */
 void (*libbalsa_progress_set_text) (LibBalsaProgress * progress,
                                     const gchar * text, guint total);
