@@ -117,7 +117,7 @@ balsa_address_book_config_new(LibBalsaAddressBook * address_book,
     }
 
     abc = g_new0(AddressBookConfig, 1);
-    g_object_set_data(G_OBJECT(address_book), "balsa-abc", abc);
+    g_object_set_data_full(G_OBJECT(address_book), "balsa-abc", abc, g_free);
     abc->address_book = address_book;
     abc->callback = callback;
     abc->type = G_TYPE_FROM_INSTANCE(address_book);
@@ -141,6 +141,7 @@ balsa_address_book_config_new_from_type(GType type,
     abc->callback = callback;
     abc->type = type;
     abc->window = create_dialog_from_type(abc);
+    g_object_weak_ref(G_OBJECT(abc->window), (GWeakNotify) g_free, abc);
 
     gtk_widget_show_all(GTK_WIDGET(abc->window));
 }
@@ -162,9 +163,10 @@ edit_book_response(GtkWidget * dialog, gint response,
         break;
     }
 
-    gtk_widget_destroy(dialog);
     if (abc->address_book)
         g_object_set_data(G_OBJECT(abc->address_book), "balsa-abc", NULL);
+
+    gtk_widget_destroy(dialog);
 }
 
 /* Radio buttons */
@@ -269,7 +271,7 @@ create_local_dialog(AddressBookConfig * abc, const gchar * type)
                                           utf8name);
         g_free(utf8name);
     }
-    g_signal_connect(G_OBJECT(dialog), "response",
+    g_signal_connect(dialog, "response",
                      G_CALLBACK(edit_book_response), abc);
 
     return dialog;
@@ -351,7 +353,7 @@ create_generic_dialog(AddressBookConfig * abc, const gchar * type)
     gtk_container_set_border_width(GTK_CONTAINER
                                    (gtk_dialog_get_content_area
                                     (GTK_DIALOG(dialog))), 12);
-    g_signal_connect(G_OBJECT(dialog), "response",
+    g_signal_connect(dialog, "response",
                      G_CALLBACK(edit_book_response), abc);
 
     return dialog;
@@ -404,7 +406,7 @@ create_osmo_dialog(AddressBookConfig *abc)
     abc->name_entry =
         libbalsa_create_grid_entry(grid, NULL, NULL, 0, name, label);
     add_radio_buttons(grid, 1, abc);
-    g_signal_connect(G_OBJECT(dialog), "response",
+    g_signal_connect(dialog, "response",
                      G_CALLBACK(edit_book_response), abc);
 
     return dialog;
@@ -500,7 +502,7 @@ create_ldap_dialog(AddressBookConfig * abc)
                                    libbalsa_address_book_get_name(LIBBALSA_ADDRESS_BOOK(ab)) : name,
 				   label);
 
-    label = libbalsa_create_grid_label(_("_Host Name"), grid, 1);
+    label = libbalsa_create_grid_label(_("_LDAP Server URI"), grid, 1);
     abc->ab_specific.ldap.host = 
 	libbalsa_create_grid_entry(grid, NULL, NULL, 1, 
 		     ab ? libbalsa_address_book_ldap_get_host(ab) : host, label);
@@ -940,45 +942,45 @@ balsa_address_book_add_menu(BalsaAddressBookCallback callback,
 
     menuitem =
         gtk_menu_item_new_with_label(_("vCard Address Book (GnomeCard)"));
-    g_signal_connect(G_OBJECT(menuitem), "activate",
+    g_signal_connect(menuitem, "activate",
                      G_CALLBACK(add_vcard_cb), abc);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
     menuitem =
         gtk_menu_item_new_with_label(_("External query (a program)"));
-    g_signal_connect(G_OBJECT(menuitem), "activate",
+    g_signal_connect(menuitem, "activate",
                      G_CALLBACK(add_externq_cb), abc);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
     menuitem = gtk_menu_item_new_with_label(_("LDIF Address Book"));
-    g_signal_connect(G_OBJECT(menuitem), "activate",
+    g_signal_connect(menuitem, "activate",
                      G_CALLBACK(add_ldif_cb), abc);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
 #ifdef ENABLE_LDAP
     menuitem = gtk_menu_item_new_with_label(_("LDAP Address Book"));
-    g_signal_connect(G_OBJECT(menuitem), "activate",
+    g_signal_connect(menuitem, "activate",
                      G_CALLBACK(add_ldap_cb), abc);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 #endif /* ENABLE_LDAP */
 
 #ifdef HAVE_SQLITE
     menuitem = gtk_menu_item_new_with_label(_("GPE Address Book"));
-    g_signal_connect(G_OBJECT(menuitem), "activate",
+    g_signal_connect(menuitem, "activate",
                      G_CALLBACK(add_gpe_cb), abc);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 #endif /* HAVE_SQLITE */
 
 #ifdef HAVE_RUBRICA
     menuitem = gtk_menu_item_new_with_label(_("Rubrica2 Address Book"));
-    g_signal_connect(G_OBJECT(menuitem), "activate",
+    g_signal_connect(menuitem, "activate",
                      G_CALLBACK(add_rubrica_cb), abc);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 #endif /* HAVE_RUBRICA */
 
 #ifdef HAVE_OSMO
     menuitem = gtk_menu_item_new_with_label(_("Osmo Address Book"));
-    g_signal_connect(G_OBJECT(menuitem), "activate",
+    g_signal_connect(menuitem, "activate",
                      G_CALLBACK(add_osmo_cb), abc);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 #endif
