@@ -32,9 +32,7 @@
 #include "balsa-mime-widget.h"
 #include "balsa-mime-widget-callbacks.h"
 #include "sendmsg-window.h"
-#ifdef HAVE_GPGME
 #include "libbalsa-gpgme.h"
-#endif
 
 typedef enum _rfc_extbody_t {
     RFC2046_EXTBODY_FTP,
@@ -77,10 +75,8 @@ static void extbody_send_mail(GtkWidget * button,
 /* message/rfc822 related stuff */
 static GtkWidget *bm_header_widget_new(BalsaMessage * bm,
 				       GtkWidget * const * buttons);
-#ifdef HAVE_GPGME
 static void add_header_sigstate(GtkGrid * grid,
 				GMimeGpgmeSigstat * siginfo);
-#endif
 
 static void bmw_message_set_headers(BalsaMessage        * bm,
                                     BalsaMimeWidget     * mw,
@@ -126,29 +122,44 @@ balsa_mime_widget_new_message(BalsaMessage * bm,
 	g_free(access_type);
     } else if (!g_ascii_strcasecmp("message/rfc822", content_type)) {
 	GtkWidget *emb_hdrs;
+	GtkWidget *widget;
+	GtkWidget *container;
 
 	mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
 
-	mw->widget = gtk_frame_new(NULL);
+	widget = gtk_frame_new(NULL);
+        gtk_container_add(GTK_CONTAINER(mw), widget);
 
-	mw->container = gtk_box_new(GTK_ORIENTATION_VERTICAL, BMW_MESSAGE_PADDING);
-	gtk_container_set_border_width(GTK_CONTAINER(mw->container),
+	container = gtk_box_new(GTK_ORIENTATION_VERTICAL, BMW_MESSAGE_PADDING);
+        balsa_mime_widget_set_container(mw, container);
+
+	gtk_container_set_border_width(GTK_CONTAINER(container),
 				       BMW_MESSAGE_PADDING);
-	gtk_container_add(GTK_CONTAINER(mw->widget), mw->container);
+	gtk_container_add(GTK_CONTAINER(widget), container);
 
-        mw->header_widget = emb_hdrs = bm_header_widget_new(bm, NULL);
-	gtk_box_pack_start(GTK_BOX(mw->container), emb_hdrs, FALSE, FALSE, 0);
+        emb_hdrs = bm_header_widget_new(bm, NULL);
+        balsa_mime_widget_set_header_widget(mw, emb_hdrs);
+
+	gtk_box_pack_start(GTK_BOX(container), emb_hdrs, FALSE, FALSE, 0);
 
         bmw_message_set_headers(bm, mw, mime_body,
-                                bm->shown_headers == HEADERS_ALL);
+                                balsa_message_get_shown_headers(bm) == HEADERS_ALL);
     } else if (!g_ascii_strcasecmp("text/rfc822-headers", content_type)) {
+	GtkWidget *widget;
+	GtkWidget *header_widget;
+
 	mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
-	mw->widget = gtk_frame_new(_("message headers"));
-	mw->header_widget = bm_header_widget_new(bm, NULL);
-        gtk_widget_set_valign(mw->header_widget, GTK_ALIGN_START);
-        gtk_widget_set_vexpand(mw->header_widget, FALSE);
-        g_object_set(G_OBJECT(mw->header_widget), "margin", 5, NULL);
-	gtk_container_add(GTK_CONTAINER(mw->widget), mw->header_widget);
+
+	widget = gtk_frame_new(_("message headers"));
+        gtk_container_add(GTK_CONTAINER(mw), widget);
+
+	header_widget = bm_header_widget_new(bm, NULL);
+        balsa_mime_widget_set_header_widget(mw, header_widget);
+
+        gtk_widget_set_valign(header_widget, GTK_ALIGN_START);
+        gtk_widget_set_vexpand(header_widget, FALSE);
+        g_object_set(header_widget, "margin", 5, NULL);
+	gtk_container_add(GTK_CONTAINER(widget), header_widget);
 	bmw_message_set_headers(bm, mw, mime_body, TRUE);
     }
 
@@ -229,21 +240,20 @@ bmw_message_extbody_url(LibBalsaMessageBody * mime_body,
 
     /* now create & return the widget... */
     mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
-    
-    mw->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, BMW_VBOX_SPACE);
-    gtk_container_set_border_width(GTK_CONTAINER(mw->widget),
+
+    gtk_container_set_border_width(GTK_CONTAINER(mw),
 				   BMW_CONTAINER_BORDER);
 
-    gtk_box_pack_start(GTK_BOX(mw->widget), gtk_label_new(msg->str), FALSE,
+    gtk_box_pack_start(GTK_BOX(mw), gtk_label_new(msg->str), FALSE,
 		       FALSE, 0);
     g_string_free(msg, TRUE);
 
     button = gtk_button_new_with_label(url);
-    gtk_box_pack_start(GTK_BOX(mw->widget), button, FALSE, FALSE,
+    gtk_box_pack_start(GTK_BOX(mw), button, FALSE, FALSE,
 		       BMW_BUTTON_PACK_SPACE);
     g_object_set_data_full(G_OBJECT(button), "call_url", url,
 			   (GDestroyNotify) g_free);
-    g_signal_connect(G_OBJECT(button), "clicked",
+    g_signal_connect(button, "clicked",
 		     G_CALLBACK(extbody_call_url), NULL);
 
     return mw;
@@ -276,21 +286,20 @@ bmw_message_extbody_mail(LibBalsaMessageBody * mime_body)
 
     /* now create & return the widget... */
     mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
-    
-    mw->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, BMW_VBOX_SPACE);
-    gtk_container_set_border_width(GTK_CONTAINER(mw->widget),
+
+    gtk_container_set_border_width(GTK_CONTAINER(mw),
 				   BMW_CONTAINER_BORDER);
 
-    gtk_box_pack_start(GTK_BOX(mw->widget), gtk_label_new(msg->str), FALSE,
+    gtk_box_pack_start(GTK_BOX(mw), gtk_label_new(msg->str), FALSE,
 		       FALSE, 0);
     g_string_free(msg, TRUE);
 
     button =
 	gtk_button_new_with_mnemonic(_
 				     ("Se_nd message to obtain this part"));
-    gtk_box_pack_start(GTK_BOX(mw->widget), button, FALSE, FALSE,
+    gtk_box_pack_start(GTK_BOX(mw), button, FALSE, FALSE,
 		       BMW_BUTTON_PACK_SPACE);
-    g_signal_connect(G_OBJECT(button), "clicked",
+    g_signal_connect(button, "clicked",
 		     G_CALLBACK(extbody_send_mail), (gpointer) mime_body);
 
 
@@ -331,6 +340,7 @@ static void
 extbody_send_mail(GtkWidget * button, LibBalsaMessageBody * mime_body)
 {
     LibBalsaMessage *message;
+    LibBalsaMessageHeaders *headers;
     LibBalsaMessageBody *body;
     gchar *data;
     GError *err = NULL;
@@ -338,9 +348,10 @@ extbody_send_mail(GtkWidget * button, LibBalsaMessageBody * mime_body)
 
     /* create a message */
     message = libbalsa_message_new();
-    message->headers->from = internet_address_list_new();
-    internet_address_list_add(message->headers->from,
-                              balsa_app.current_ident->ia);
+    headers = libbalsa_message_get_headers(message);
+    headers->from = internet_address_list_new();
+    internet_address_list_add(headers->from,
+                              libbalsa_identity_get_address(balsa_app.current_ident));
 
     data = libbalsa_message_body_get_parameter(mime_body, "subject");
     if (data) {
@@ -349,7 +360,7 @@ extbody_send_mail(GtkWidget * button, LibBalsaMessageBody * mime_body)
     }
 
     data = libbalsa_message_body_get_parameter(mime_body, "server");
-    message->headers->to_list = internet_address_list_parse_string(data);
+    headers->to_list = internet_address_list_parse_string(data);
     g_free(data);
 
     /* the original body my have some data to be returned as commands... */
@@ -379,7 +390,8 @@ extbody_send_mail(GtkWidget * button, LibBalsaMessageBody * mime_body)
     libbalsa_message_append_part(message, body);
     result = libbalsa_message_send(message, balsa_app.outbox, NULL,
 				   balsa_find_sentbox_by_url,
-				   balsa_app.current_ident->smtp_server,
+				   libbalsa_identity_get_smtp_server
+                                   (balsa_app.current_ident),
 				   balsa_app.send_progress_dialog,
                                    GTK_WINDOW(gtk_widget_get_toplevel
                                               (button)),
@@ -389,7 +401,7 @@ extbody_send_mail(GtkWidget * button, LibBalsaMessageBody * mime_body)
 			     _("Sending the external body request failed: %s"),
 			     err ? err->message : "?");
     g_error_free(err);
-    g_object_unref(G_OBJECT(message));
+    g_object_unref(message);
 }
 
 
@@ -399,19 +411,23 @@ BalsaMimeWidget *
 balsa_mime_widget_new_message_tl(BalsaMessage * bm,
                                  GtkWidget * const * tl_buttons)
 {
-    GtkWidget *headers;
     BalsaMimeWidget *mw;
+    GtkWidget *headers;
+    GtkWidget *container;
 
     mw = g_object_new(BALSA_TYPE_MIME_WIDGET, NULL);
 
-    mw->widget = gtk_box_new(GTK_ORIENTATION_VERTICAL, BMW_MESSAGE_PADDING);
-    gtk_container_set_border_width(GTK_CONTAINER(mw->widget), BMW_MESSAGE_PADDING);
+    gtk_container_set_border_width(GTK_CONTAINER(mw), BMW_MESSAGE_PADDING);
 
-    mw->header_widget = headers = bm_header_widget_new(bm, tl_buttons);
-    gtk_box_pack_start(GTK_BOX(mw->widget), headers, FALSE, FALSE, 0);
+    headers = bm_header_widget_new(bm, tl_buttons);
+    balsa_mime_widget_set_header_widget(mw, headers);
 
-    mw->container = gtk_box_new(GTK_ORIENTATION_VERTICAL, BMW_MESSAGE_PADDING);
-    gtk_box_pack_start(GTK_BOX(mw->widget), mw->container, TRUE, TRUE,
+    gtk_box_pack_start(GTK_BOX(mw), headers, FALSE, FALSE, 0);
+
+    container = gtk_box_new(GTK_ORIENTATION_VERTICAL, BMW_MESSAGE_PADDING);
+    balsa_mime_widget_set_container(mw, container);
+
+    gtk_box_pack_start(GTK_BOX(mw), container, TRUE, TRUE,
 		       BMW_CONTAINER_BORDER - BMW_MESSAGE_PADDING);
 
     return mw;
@@ -440,7 +456,7 @@ bm_header_extend_popup(GtkWidget * widget, GtkMenu * menu, gpointer arg)
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), separator);
     gtk_widget_show(separator);
     menu_item = gtk_menu_item_new_with_label(_("Reply…"));
-    g_signal_connect(G_OBJECT(menu_item), "activate",
+    g_signal_connect(menu_item, "activate",
                      G_CALLBACK(bm_header_ctx_menu_reply),
                      arg);
     gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
@@ -508,11 +524,13 @@ bm_header_widget_new(BalsaMessage * bm, GtkWidget * const * buttons)
     gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area),
                               GTK_BUTTONBOX_START);
 #endif                          /* GTK_INFO_BAR_WRAPPING_IS_BROKEN */
-    if (!bm->face_box) {
-        bm->face_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
-        gtk_container_add(GTK_CONTAINER(action_area), bm->face_box);
+    if (balsa_message_get_face_box(bm) == NULL) {
+        GtkWidget *face_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+
+        balsa_message_set_face_box(bm, face_box);
+        gtk_container_add(GTK_CONTAINER(action_area), face_box);
         gtk_button_box_set_child_non_homogeneous(GTK_BUTTON_BOX(action_area),
-                                                 bm->face_box, TRUE);
+                                                 face_box, TRUE);
     }
 
     if (buttons) {
@@ -665,7 +683,7 @@ add_header_address_list(BalsaMessage * bm, GtkGrid * grid,
     if (list == NULL || internet_address_list_length(list) == 0)
 	return;
 
-    if (!(bm->shown_headers == HEADERS_ALL ||
+    if (!(balsa_message_get_shown_headers(bm) == HEADERS_ALL ||
 	  libbalsa_find_word(header, balsa_app.selected_headers)))
 	return;
 
@@ -702,7 +720,7 @@ bmw_message_set_headers_d(BalsaMessage           * bm,
     gchar *date;
     GtkWidget * widget;
 
-    if (!(widget = mw->header_widget))
+    if ((widget = balsa_mime_widget_get_header_widget(mw)) == NULL)
 	return;
 
     grid = bm_header_widget_get_grid(widget);
@@ -717,14 +735,12 @@ bmw_message_set_headers_d(BalsaMessage           * bm,
         return;
     }
 
-    if (bm->shown_headers == HEADERS_NONE) {
-        g_signal_connect(G_OBJECT(widget), "realize",
+    if (balsa_message_get_shown_headers(bm) == HEADERS_NONE) {
+        g_signal_connect(widget, "realize",
                          G_CALLBACK(gtk_widget_hide), NULL);
 	return;
     }
 
-    bm->tab_position = 0;
-
     add_header_gchar(grid, "subject", _("Subject:"), subject,
                      show_all_headers);
 
@@ -779,7 +795,6 @@ bmw_message_set_headers_d(BalsaMessage           * bm,
 	g_free(hdr);
     }
 
-#ifdef HAVE_GPGME
     if (part) {
 	if (libbalsa_message_body_multipart_signed(part)) {
 	    /* top-level part is RFC 3156 or RFC 8551 multipart/signed */
@@ -789,7 +804,6 @@ bmw_message_set_headers_d(BalsaMessage           * bm,
 	    add_header_sigstate(grid, part->sig_info);
 	}
     }
-#endif
 }
 
 static void
@@ -804,8 +818,10 @@ bmw_message_set_headers(BalsaMessage        * bm,
     bmw_message_set_headers_d(bm, mw, part->embhdrs, part->parts,
                               part->embhdrs ? part->embhdrs->subject : NULL,
                               show_all_headers);
-    if (!(widget = mw->header_widget))
+
+    if ((widget = balsa_mime_widget_get_header_widget(mw)) == NULL)
 	return;
+
     grid = bm_header_widget_get_grid(widget);
     gtk_container_foreach(GTK_CONTAINER(grid), (GtkCallback) foreach_label,
                           part);
@@ -817,7 +833,7 @@ balsa_mime_widget_message_set_headers(BalsaMessage        * bm,
                                       LibBalsaMessageBody * part)
 {
     bmw_message_set_headers(bm, mw, part,
-                            bm->shown_headers == HEADERS_ALL);
+                            balsa_message_get_shown_headers(bm) == HEADERS_ALL);
 }
 
 void
@@ -828,10 +844,9 @@ balsa_mime_widget_message_set_headers_d(BalsaMessage           * bm,
                                         const gchar            * subject)
 {
     bmw_message_set_headers_d(bm, mw, headers, part, subject,
-                              bm->shown_headers == HEADERS_ALL);
+                              balsa_message_get_shown_headers(bm) == HEADERS_ALL);
 }
 
-#ifdef HAVE_GPGME
 /*
  * Add the short status of a signature info siginfo to the message headers in
  * view
@@ -845,7 +860,7 @@ add_header_sigstate(GtkGrid * grid, GMimeGpgmeSigstat * siginfo)
     GtkWidget *label;
 
     info_str = g_mime_gpgme_sigstat_info(siginfo, TRUE);
-    format = (siginfo->status == GPG_ERR_NO_ERROR) ? "<i>%s</i>" : "<b><i>%s</i></b>";
+    format = (g_mime_gpgme_sigstat_status(siginfo) == GPG_ERR_NO_ERROR) ? "<i>%s</i>" : "<b><i>%s</i></b>";
     msg = g_markup_printf_escaped(format, info_str);
     g_free(info_str);
 
@@ -857,4 +872,3 @@ add_header_sigstate(GtkGrid * grid, GMimeGpgmeSigstat * siginfo)
 
     gtk_grid_attach_next_to(grid, label, NULL, GTK_POS_BOTTOM, 2, 1);
 }
-#endif
