@@ -99,7 +99,7 @@ mw_set_active(MessageWindow * mw,
 static void
 mw_set_part_buttons_sensitive(MessageWindow * mw, BalsaMessage * msg)
 {
-    if (!msg || !msg->treeview)
+    if (msg == NULL || balsa_message_get_tree_view(msg) == NULL)
 	return;
 
     mw_set_enabled(mw, "next-part",
@@ -116,7 +116,9 @@ message_window_idle_handler(MessageWindow * mw)
     mw->idle_handler_id = 0;
 
     msg = BALSA_MESSAGE(mw->bmessage);
-    if (!balsa_message_set(msg, mw->message->mailbox, mw->message->msgno)) {
+    if (!balsa_message_set(msg,
+                           libbalsa_message_get_mailbox(mw->message),
+                           libbalsa_message_get_msgno(mw->message))) {
         gtk_widget_destroy(mw->window);
         return FALSE;
     }
@@ -304,12 +306,13 @@ mw_menubar_foreach(GtkWidget *widget, gpointer data)
 static void
 mw_set_buttons_sensitive(MessageWindow * mw)
 {
-    LibBalsaMailbox *mailbox = mw->message->mailbox;
+    LibBalsaMailbox *mailbox;
     BalsaIndex *index = mw->bindex;
-    guint current_msgno = mw->message->msgno;
+    guint current_msgno = libbalsa_message_get_msgno(mw->message);
     gboolean enable;
 
-    if (!mailbox) {
+    mailbox = libbalsa_message_get_mailbox(mw->message);
+    if (mailbox == NULL) {
         gtk_widget_destroy(mw->window);
         return;
     }
@@ -320,12 +323,12 @@ mw_set_buttons_sensitive(MessageWindow * mw)
     enable = index && balsa_index_previous_msgno(index, current_msgno) > 0;
     mw_set_enabled(mw, "previous-message", enable);
 
-    enable = index && index->mailbox_node->mailbox->unread_messages > 0;
+    mailbox = index != NULL ? balsa_index_get_mailbox(index): NULL;
+
+    enable = mailbox != NULL && libbalsa_mailbox_get_unread_messages(mailbox) > 0;
     mw_set_enabled(mw, "next-unread", enable);
 
-    enable = index
-        && libbalsa_mailbox_total_messages(index->mailbox_node->mailbox) >
-        0;
+    enable = mailbox != NULL && libbalsa_mailbox_total_messages(mailbox) > 0;
     mw_set_enabled(mw, "next-flagged", enable);
 }
 
@@ -359,7 +362,7 @@ destroy_message_window(GtkWidget * widget, MessageWindow * mw)
 {
     if (mw->bindex) {           /* BalsaIndex still exists */
         g_object_weak_unref(G_OBJECT(mw->bindex), mw_bindex_closed_cb, mw);
-        g_signal_handlers_disconnect_matched(G_OBJECT(mw->bindex),
+        g_signal_handlers_disconnect_matched(mw->bindex,
                                              G_SIGNAL_MATCH_DATA, 0, 0,
                                              NULL, NULL, mw);
         mw->bindex = NULL;
@@ -367,14 +370,14 @@ destroy_message_window(GtkWidget * widget, MessageWindow * mw)
 
     if (mw->mailbox) {
         g_object_remove_weak_pointer(G_OBJECT(mw->mailbox), (gpointer *) &mw->mailbox);
-        g_signal_handlers_disconnect_matched(G_OBJECT(mw->mailbox),
+        g_signal_handlers_disconnect_matched(mw->mailbox,
                                              G_SIGNAL_MATCH_DATA, 0, 0,
                                              NULL, NULL, mw);
         mw->mailbox = NULL;
     }
 
     if (mw->bmessage)
-        g_signal_handlers_disconnect_matched(G_OBJECT(mw->bmessage),
+        g_signal_handlers_disconnect_matched(mw->bmessage,
                                              G_SIGNAL_MATCH_DATA, 0, 0,
                                              NULL, NULL, mw);
 
@@ -387,7 +390,7 @@ destroy_message_window(GtkWidget * widget, MessageWindow * mw)
 static void
 mw_expunged_cb(LibBalsaMailbox * mailbox, guint msgno, MessageWindow * mw)
 {
-    if (mw->message && (guint) mw->message->msgno == msgno)
+    if (mw->message != NULL && (guint) libbalsa_message_get_msgno(mw->message) == msgno)
         gtk_widget_destroy(mw->window);
 }
 
@@ -399,7 +402,8 @@ mw_reply_activated(GSimpleAction * action, GVariant * parameter,
 
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_reply(mw->message->mailbox, mw->message->msgno,
+    sendmsg_window_reply(libbalsa_message_get_mailbox(mw->message),
+                         libbalsa_message_get_msgno(mw->message),
                          SEND_REPLY);
 }
 
@@ -411,7 +415,8 @@ mw_reply_all_activated(GSimpleAction * action, GVariant * parameter,
 
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_reply(mw->message->mailbox, mw->message->msgno,
+    sendmsg_window_reply(libbalsa_message_get_mailbox(mw->message),
+                         libbalsa_message_get_msgno(mw->message),
                          SEND_REPLY_ALL);
 }
 
@@ -423,7 +428,8 @@ mw_reply_group_activated(GSimpleAction * action, GVariant * parameter,
 
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_reply(mw->message->mailbox, mw->message->msgno,
+    sendmsg_window_reply(libbalsa_message_get_mailbox(mw->message),
+                         libbalsa_message_get_msgno(mw->message),
                          SEND_REPLY_GROUP);
 }
 
@@ -435,7 +441,9 @@ mw_forward_attached_activated(GSimpleAction * action, GVariant * parameter,
 
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_forward(mw->message->mailbox, mw->message->msgno, TRUE);
+    sendmsg_window_forward(libbalsa_message_get_mailbox(mw->message),
+                           libbalsa_message_get_msgno(mw->message),
+                           TRUE);
 }
 
 static void
@@ -446,7 +454,8 @@ mw_forward_inline_activated(GSimpleAction * action, GVariant * parameter,
 
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_forward(mw->message->mailbox, mw->message->msgno,
+    sendmsg_window_forward(libbalsa_message_get_mailbox(mw->message),
+                           libbalsa_message_get_msgno(mw->message),
                            FALSE);
 }
 
@@ -459,7 +468,8 @@ mw_forward_default_activated(GSimpleAction * action, GVariant * parameter,
 
     g_return_if_fail(mw != NULL);
 
-    sendmsg_window_forward(mw->message->mailbox, mw->message->msgno,
+    sendmsg_window_forward(libbalsa_message_get_mailbox(mw->message),
+                           libbalsa_message(get_msgno(mw->message),
                            balsa_app.forward_attached);
 }
 #endif
@@ -548,10 +558,10 @@ mw_set_selected(MessageWindow * mw, void (*select_func) (BalsaIndex *))
     LibBalsaMessage *message;
     MessageWindow *tmp;
 
-    balsa_index_set_next_msgno(mw->bindex, mw->message->msgno);
+    balsa_index_set_next_msgno(mw->bindex, libbalsa_message_get_msgno(mw->message));
     select_func(mw->bindex);
     msgno = balsa_index_get_next_msgno(mw->bindex);
-    message = libbalsa_mailbox_get_message(mw->message->mailbox, msgno);
+    message = libbalsa_mailbox_get_message(libbalsa_message_get_mailbox(mw->message), msgno);
     if (!message)
         return;
 
@@ -574,16 +584,18 @@ static void
 message_window_move_message(MessageWindow * mw, LibBalsaMailbox * mailbox)
 {
     GArray *messages;
+    guint msgno;
     LibBalsaMessage *original = mw->message;
 
     g_return_if_fail(mailbox != NULL);
 
     /* Transferring to same mailbox? */
-    if (mw->message->mailbox == mailbox)
+    if (libbalsa_message_get_mailbox(mw->message) == mailbox)
         return;
 
     messages = g_array_new(FALSE, FALSE, sizeof(guint));
-    g_array_append_val(messages, mw->message->msgno);
+    msgno = libbalsa_message_get_msgno(mw->message);
+    g_array_append_val(messages, msgno);
 
     if (balsa_app.mw_action_after_move == NEXT_UNREAD)
         /* Try selecting the next unread message. */
@@ -719,14 +731,18 @@ mw_select_part_cb(BalsaMessage * bm, gpointer data)
 #endif                          /* HAVE_HTML_WIDGET */
 
     /* set window title */
-    if (bm && bm->message) {
-        from = internet_address_list_to_string(bm->message->headers->from,
-                                               FALSE);
-        title = g_strdup_printf(_("Message from %s: %s"), from,
-                                LIBBALSA_MESSAGE_GET_SUBJECT(bm->message));
-        g_free(from);
-        gtk_window_set_title(GTK_WINDOW(mw->window), title);
-        g_free(title);
+    if (bm != NULL) {
+        LibBalsaMessage *message = balsa_message_get_message(bm);
+
+        if (message != NULL) {
+            from = internet_address_list_to_string(libbalsa_message_get_headers(message)->from,
+                                                   FALSE);
+            title = g_strdup_printf(_("Message from %s: %s"), from,
+                                    LIBBALSA_MESSAGE_GET_SUBJECT(message));
+            g_free(from);
+            gtk_window_set_title(GTK_WINDOW(mw->window), title);
+            g_free(title);
+        }
     }
 
     mw_set_part_buttons_sensitive(mw, bm);
@@ -843,12 +859,12 @@ message_window_new(LibBalsaMailbox * mailbox, guint msgno)
 
     gtk_window_set_role(GTK_WINDOW(window), "message");
 
-    g_signal_connect(G_OBJECT(window), "destroy",
+    g_signal_connect(window, "destroy",
 		     G_CALLBACK(destroy_message_window), mw);
-    
+
     mw->bindex = balsa_find_index_by_mailbox(mailbox);
     g_object_weak_ref(G_OBJECT(mw->bindex), mw_bindex_closed_cb, mw);
-    g_signal_connect_swapped(G_OBJECT(mw->bindex), "index-changed",
+    g_signal_connect_swapped(mw->bindex, "index-changed",
 			     G_CALLBACK(mw_set_buttons_sensitive), mw);
 
     mw->mailbox = mailbox;
@@ -863,14 +879,14 @@ message_window_new(LibBalsaMailbox * mailbox, guint msgno)
                           &move_menu);
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(move_menu), submenu);
 
-    if (mailbox->readonly) {
+    if (libbalsa_mailbox_get_readonly(mailbox)) {
 	gtk_widget_set_sensitive(move_menu, FALSE);
 	mw_disable_trash(mw);
     }
     if (mailbox == balsa_app.trash)
 	mw_disable_trash(mw);
     mw->bmessage = balsa_message_new();
-    
+
     gtk_box_pack_start(GTK_BOX(vbox), mw->bmessage, TRUE, TRUE, 0);
     gtk_widget_show(vbox);
 
