@@ -24,6 +24,6 @@
 #include "libbalsa.h"
 
 void balsa_store_address_from_messages(GList * messages);
-void balsa_store_address(const LibBalsaAddress *address);
+void balsa_store_address(LibBalsaAddress *address);
 
 #endif
